#!/usr/bin/perl -w

=head1 NAME

B<POWER::LOG> -  

=head1 SYNOPSIS

 use POWER::LOG;

 $LOG = POWER::LOG->new();
 $LOG->config({
    OUTPUT  => \*STDERR,
    PREFIX  => "%D %T %P::%F() : ",
    LEVEL   => "WARN",
    });
 # OR
 $LOG = $POWER::LOG::DEFAULT;
 $LOG->config({
    OUTPUT  => \*STDERR,
    LEVEL   => "WARN",
    });
 
 $LOG->ERR($message);
 $LOG->WARN($message);
 $LOG->INFO($message);
 $LOG->DEBUG($message);
 $LOG->DUMP($message);

=head1 DESCRIPTION

      .
       .
     {LEVEL}   
.

=head1 METHODS

=over 4

=cut

package POWER::LOG;

use POWER::lib qw(time);

use strict;
BEGIN { vars qw($VERSION $DEFAULT $LEVEL) }

$VERSION = "2.12";
$DEFAULT = POWER::LOG->new({PREFIX=>'%S %L %_',OUTPUT=>\*STDERR,LEVEL=>"ERR"});

=item B<new>( [\%config] )

         .
 $config,   ,   config().

 new()     POWER::LOG,   
.

=cut

sub new {
    my $this  = shift;
    my $class = ref($this) || $this ;
    my $self  = { CONFIG => {}, };
    bless $self, $class ;
    $self->config($this->config()) if ref($this);
    $self->config(@_);
    return $self;
}

=item B<config>( [\%config] )

  .
$config -      .
    ,   B<CONFIG>.

        .

=cut

sub config {
    my ($self, $config) = (shift, @_);
    $config = {} if ref($config) ne "HASH";
    UpdateHash($self->{CONFIG}, $config);
    $self->{skip} = ref($self->{CONFIG}{OUTPUT}) ne "GLOB";
    @{$self}{"ERR","WARN","INFO","DEBUG","DUMP"} = 
        $_ eq "ERR"  ? (1,0,0,0,0) : $_ eq "WARN"  ? (1,1,0,0,0) : 
        $_ eq "INFO" ? (1,1,1,0,0) : $_ eq "DEBUG" ? (1,1,1,1,0) :
        $_ eq "DUMP" ? (1,1,1,1,1) : (0,0,0,0,0) for $self->{CONFIG}{LEVEL};
    my @pfx = split /(\%.)/, $self->{CONFIG}{PREFIX};
    my %need;
    for (@pfx) {
        /^\%\%$/ && do{$_='"%"'}
     || /^\%L/ && do{$_='$LEVEL'}
     || /^\%S/ && do{$_='$data{S}'; $need{Sec}      = 1 }
     || /^\%D/ && do{$_='$data{D}'; $need{DateTime} = 1 }
     || /^\%T/ && do{$_='$data{T}'; $need{DateTime} = 1 }
     || /^\%P/ && do{$_='$data{P}'; $need{Caller}   = 1 }
     || /^\%F/ && do{$_='$data{F}'; $need{Caller}   = 1; $need{Func}    = 1 }
     || /^\%_/ && do{$_='$data{_}'; $need{Stack}    = 1 }
     ||           do{$_="\"\Q$_\E\""};
    }
    my $code = '{ my (%data, $t, @t, $was, $now); sub {';
    $code .= '
        $t = time();
        $data{S} = sprintf \'%d.%05d\', $t, ($t-int($t))*100000;' if $need{Sec};
    $code .= '
        if (($now = CORE::time()) != $was) {
            ($was, @t) = ($now, localtime());
            $data{D} = sprintf \'%02d/%02d/%04d\', $t[3], $t[4]+1, $t[5]+1900;
            $data{T} = sprintf \'%02d:%02d:%02d\', $t[2], $t[1], $t[0];
        }' if $need{DateTime};
    $code .= '
        $data{P} = caller(1);' if $need{Caller};
    $code .= '
        $data{F} = substr((caller(2))[3], length($data{P})+2);' if $need{Func};
    $code .= '
        $data{_}=3; 1 while caller($data{_}++); $data{_} = " " x ($data{_}-3);'
        if $need{Stack};
    $code .= 'return ('.join(',',grep {!/^\"\"$/} @pfx).'); }}';
    $self->{prefix} = eval $code;
}

=item B<ERR>( $message )

     . [ ]

=item B<WARN>( $message )

     . [  ]

=item B<INFO>( $message )

   . [   /  ]

=item B<DEBUG>( $message )

    . [   ]

=item B<DUMP>( $message )

       .

=cut

{ local $LEVEL;
sub ERR     { $LEVEL = "ERR";   goto &_log if $_[0]->{$LEVEL} }
sub WARN    { $LEVEL = "WARN";  goto &_log if $_[0]->{$LEVEL} }
sub INFO    { $LEVEL = "INFO";  goto &_log if $_[0]->{$LEVEL} }
sub DEBUG   { $LEVEL = "DEBUG"; goto &_log if $_[0]->{$LEVEL} }
sub DUMP    { $LEVEL = "DUMP";  goto &_log if $_[0]->{$LEVEL} }
sub _log {
    print {$_[0]->{CONFIG}{OUTPUT}} $_[0]->{prefix}->(), $_[1], "\n"
        unless $_[0]->{skip};
}}

=back

=head1 CONFIG

=over 4

=item B<{OUTPUT}>

 ,     .   ,
    .

=item B<{PREFIX}>

,        .
 {PREFIX}     ,   
   .     printf(),  
,     :

 %L - LEVEL,    
 %S -      
 %D -    :  "//"
 %T -    : "::"
 %P -   : "main"  "POWER::NB::IO"
 %F -   :   "my_func_name"
 %_ - X ,  X -  
 %% -  "%"

 : "%D %T %P::%F() "

=item B<{LEVEL}>

 {LEVEL}        : "ERR","WARN","INFO",
"DEBUG","DUMP"     .   
,       {LEVEL}. , 
{LEVEL}  "INFO"     "ERR", "WARN"  "INFO".

=back

=head1 SPEED

 POWER::LOG      
 1000    .

       -
 : %L, %P, %%; : %S, %D, %T; : %F, %_.

   ({OUTPUT}    ),  
 .

=head1 HISTORY

 2.12          9 04:19:21 EEST 2002
        -       10 
 2.11         27 16:36:06 EET 2002
        -  : %L, %S, %_
        - DEFAULT   {OUTPUT=>\*STDERR,LEVEL=>"ERR"}
 2.10         25 10:41:50 EET 2002
        -  $POWER::LOG::DEFAULT
 2.01
 2.00         10 05:42:11 EET 2002
        -    
 1.01         28 21:28:52 EET 2001
        -   /
 1.00         28 16:48:16 EET 2001
        -     ( print)

=head1 AUTHOR

Alex Efros <powerman@asdfGroup.com>

=cut

1;
